// Programme permettant la conversion des entres analogique
// du 16F877 et de transmettre les valeurs par le port srie
//----------------------------------------------------------

// Version 1.2

#include <pic.h>

// Declaration des Variables
//----------------------------------------------------------
char buf[8];

unsigned char
	udata, // Charactre reu de l'USART
	mux	// bank A/D utilis
;

unsigned int AD;
unsigned long ADTemp; //Stocke la valeur d'une voie

// Prototypes
//----------------------------------------------------------
void putrsUSART(const char *data);
void Setup(void);
void temp(unsigned int delay);
void adc_read(unsigned char channel);



// Fonctions
//----------------------------------------------------------



//---------------------------------------------------------------------
// Setup() initialise les variables du programme
//---------------------------------------------------------------------
void Setup(void)
{
	udata = RCREG;
	udata = RCREG;
	RCREG = 0;
	udata = 0;
	SPBRG = 10; // 115200 baud @ 20MHz
	TXSTA = 0x24; // setup USART transmit
	RCSTA = 0x90; // setup USART receive
	TRISA = 0xFF; // PORT A en entre
	TRISE = 0x07; // PORT E en entre
	TRISB = 0x00; // PORT B en sortie;
	ADCON1 = 0x80; // Setup A/D converter
	ADCON0 = 0x81; // Configure la frquence  Fosc/32 et active les entres analogiques
}

//-----------------------------------------------------------------------
// putrsUSART()
//
// Envoie une chane de caractre de la mmoire jusqu' l'interface srie
//-----------------------------------------------------------------------
void putrsUSART(const char *data)
{
	do
	{
		while(!(TXSTA & 0x02));
		TXREG = *data;
	} while( *data++ );
}

//----------------------------------------------------------------------
// Main()
//----------------------------------------------------------------------
main() {
	Setup();
	
	while(1) {
		char channel; // Channel numris
		int CRC; // Valeur du CRC
		
		CLRWDT();
		
		// Multiplexage des entres analogiques
		for(mux=0;mux<4;mux++) {
			PORTB=mux;
			for(channel=2;channel<8;channel++) {	
			
   			    adc_read(channel); // Lit un canal
				AD= (ADRESH*256 + ADRESL); // Met dans ADTemp les 10 premiers bit rsultant de la conversion
			
				temp(10); // Tempo de stabilisation du port srie
				
				// Calcul du CRC
				CRC=(((mux*8)+channel+1)+AD)%255;
					
				// envoie sur le port srie
				buf[0]=(mux*8)+channel+1;
				buf[1]=0x2C;
				buf[2]=AD/256;
				buf[3]=AD;
				buf[4]=CRC;
				buf[5]=0;
				putrsUSART(buf); // Envoie la chaine sur USART
			}
		}
	} //end while(1)
}

//---------------------------------------------------------
// Temp(int delay);
// Temporaisation
//---------------------------------------------------------

void temp(unsigned int delay) {
	unsigned int j;
	for(j=0;j<delay;j++);
}

//---------------------------------------------------------
// adc_read(unsigned char channel)
// Fonction qui permet la lecture des canaux analogiques
//---------------------------------------------------------
void adc_read(unsigned char channel)
{
	ADCON0 = (channel << 3) + 0x81;		// enable ADC, Fosc/32.
	temp(15); // temps d'attente ncessaire pour l'initialisation de la conversion
	ADGO = 1;
	while(ADGO)
		continue;	// Attend qu'une conversion soit termin
}
